/*
 ******************************************************************************
 *
 * Copyright Ben Suffolk, 2004/5
 *
 * Ben@vanilla.net
 *
 * You may not reverse engineer, decompile or disassemble the software, except
 * and only to the extent that such activity is expressly permitted by
 * applicable law notwithstanding this limitation.
 *
 * You may not sell, rent, lease, sub-lease or incorporate the whole or any
 * part of the use of the software in any other program or system without the
 * express written agreement
 *
 ******************************************************************************
 *
 * See the header comments in HWInterface.m for details
 *
 ******************************************************************************
 *
 * Last Author:   $Author: suffo_b $
 * Last modified: $Date: 2005/03/21 09:06:22 $
 * Revision:      $Revision: 1.1 $
 *
 ******************************************************************************
*/

#import <Foundation/Foundation.h>
#import <AppKit/NSProgressIndicator.h>

/* Base path for all Interfaces */
#define HW_DIR @"/dev/"
#define HW_BASE @"cu.usbserial-DMX"

/* Inbound message modes */
#define MODE_CMD   0x00
#define MODE_VER   0x01
#define MODE_STR   0x02
#define MODE_ADDR  0x03
#define MODE_VAL   0x04

/* Commands */
#define CMD_RSET    0x22
#define CMD_VER     0x24
#define CMD_NOP     0x26
#define CMD_TX_SSC  0x42
#define CMD_TX_ON   0x44
#define CMD_TX_OFF  0x46
#define CMD_TX_SLC  0x4E
#define CMD_CH_SET  0x48
#define CMD_BO_ON   0x4A
#define CMD_BO_OFF  0x4C
#define CMD_RXSC    0x62
#define CMD_RXON    0x64
#define CMD_RXOFF   0x66
#define CMD_MON_LC  0x68
#define CMD_MON_FPS 0x6A
#define CMD_MON_SC  0x6C

/* Command responses */
#define RESP_USAVE  0x84
#define RESP_RSET   0xA2
#define RSP_NORST   0xA3
#define RESP_VER    0xA4
#define RESP_NOP    0xA6
#define RSP_TX_SSC  0xC2
#define RSP_TX_ON   0xC4
#define RSP_TX_OFF  0xC6
#define RSP_CH_SET  0xC8
#define RSP_BO_ON   0xCA
#define RSP_BO_OFF  0xCC
#define RSP_TX_SLC  0xCE
#define RESP_RXSC   0xE2
#define RESP_RXON   0xE4
#define RESP_RXOFF  0xE6
#define RSP_MON_LC  0xE8
#define RSP_MON_LCH 0xE9
#define RSP_MON_FPS 0xEA
#define RSP_MON_SC  0xEC

/* System generated responses */
#define RESP_DMXVAL  0xFE
#define RESP_DMXVALH 0xFF
#define RESP_DMXERR  0xFC
#define RESP_HWERR   0xFA
#define RESP_NO_DMX  0xF8
#define RESP_DMX     0xF6
#define RESP_NO_MON  0xF4


/**
 *
 * DMX Response Object
 *
 * This object is used to hold inbound DMX Values, and other responses
 *
 **/
@interface DMXResponse : NSObject
 {
 unsigned char code;
 unsigned int channel;
 unsigned char value;
 }

/* Constructor and accessor methods */
- (id)initWithCode:(unsigned char)aCode channel:(unsigned int)aChannel value:(unsigned char)aValue;
- (id)initWithChannel:(unsigned int)aChannel value:(unsigned char)aValue;
- (unsigned char)getCode;
- (unsigned int)getChannel;
- (unsigned char)getValue;

@end


/**
 *
 * DMX Version Object
 *
 * This object is used to hold version information about the USB / DMX Interface
 *
 **/
@interface DMXVersion : NSObject
 {
 int major;
 int minor;
 NSString *string;
 }

/* Constructor and accessor methods */
- (id)initWithMajor:(int)aMajor minor:(int)aMinor string:(char *)str;
- (int)getMajor;
- (int)getMinor;
- (NSString *)getString;

@end


/**
 *
 * Hardware Controller Object
 *
 * This object is used to access the USB / DMX Interface.
 *
 **/
@interface HWInterface : NSObject
 {
 /* Path to the interface, and file descriptor for it */
 NSString *path;
 int fd;

 /* Used for the receiver thread */
 BOOL running;
 NSThread *receiver;
 NSLock *threadLock;
 
 /* Our own copy of the TX DMX universe */
 unsigned char out_universe[512];
 }

/* Static methods */
+ (NSArray *)interfacePaths;
+ (char *)lookupResponse:(unsigned char)aResponse;

/* Constructors */
- (id)init;
- (id)initWithPath:(NSString *)aPath;

/* Send Commands */
- (void)SendCommand:(unsigned char)aCommand;
- (void)SendCommand:(unsigned char)aCommand channel:(unsigned int)aChannel;
- (void)SendCommand:(unsigned char)aCommand value:(unsigned char)aValue;
- (void)SendCommand:(unsigned char)aCommand channel:(unsigned int)aChannel value:(unsigned char)aValue;

/* Internal methods */
- (void)receiverThread:(id)anObject;
- (void)postDebug:(NSString *)debug;
- (void)dealloc;


@end
